"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.express = void 0;
var code_builder_1 = require("../../../../helpers/code-builder");
exports.express = {
    info: {
        key: 'express',
        title: 'Express',
        link: 'https://expressjs.com/',
        description: 'ReadMe Metrics Webhooks SDK usage on Express'
    },
    convert: function (_a, options) {
        var security = _a.security, server = _a.server;
        var opts = __assign({ indent: '  ' }, options);
        var _b = new code_builder_1.CodeBuilder({ indent: opts.indent }), blank = _b.blank, join = _b.join, push = _b.push, ranges = _b.ranges, variable = _b.variable;
        push('// Save this code as `server.js`');
        push('// Run the server with `node server.js`');
        push("const readme = require('readmeio');");
        push("const express = require('express');");
        blank();
        push('const app = express();');
        blank();
        push("app.post('/webhook', express.raw({ type: 'application/json' }), (req, res) => {");
        push('// Verify the request is legitimate and came from ReadMe', 1);
        push("const signature = req.headers['readme-signature'];", 1);
        blank();
        push('// Your ReadMe secret', 1);
        push("const secret = 'rdme_xxxx';", 1); // TODO should pass in JWT secret here
        blank();
        push('try {', 1);
        push('readme.verify(req.body, signature, secret);', 2);
        push('} catch (e) {', 1);
        push('// Handle invalid requests', 2);
        push('return res.sendStatus(401);', 2);
        push('}', 1);
        blank();
        push('// Fetch the user from the db', 1);
        push('db.find({ email: req.body.email }).then(user => {', 1);
        push('return res.json({', 2);
        if (server.length) {
            push('// OAS Server variables', 3);
            server.forEach(function (data) {
                push("".concat(data.name, ": '").concat(data["default"] || data["default"] === '' ? data["default"] : data.name, "',"), 3);
                variable('server', data.name);
            });
        }
        if (server.length && security.length) {
            blank();
        }
        if (security.length) {
            push('// OAS Security variables', 3);
            security.forEach(function (data) {
                if (data.type === 'http') {
                    // Only HTTP Basic auth has any special handling for supplying auth.
                    if (data.scheme === 'basic') {
                        push("".concat(data.name, ": { user: 'user', pass: 'pass' },"), 3);
                        variable('security', data.name);
                        return;
                    }
                }
                push("".concat(data.name, ": '").concat(data["default"] || data["default"] === '' ? data["default"] : data.name, "',"), 3);
                variable('security', data.name);
            });
        }
        push('});', 2);
        push('});', 1);
        push('});');
        return {
            ranges: ranges(),
            snippet: join()
        };
    }
};
